#!/usr/local/bin/vm shell -S /bin/bash

#
#This is a simple script to answer the phone.
#
# 

# ============================================================
# CONFIGURATION SECTION
# ============================================================
OUTGOING_DIR=/var/spool/voice/messages
INCOMING_DIR=/var/spool/voice/incoming
STARTING_DATETIME=`/bin/date +%Y%b%d_%H-%M-%S%Z`
PATH=$PATH:/usr/local/bin
MESSAGES_DIR=/var/spool/voice/messages
FLAG_FILE=/var/spool/voice/newmail.flag

#the users for each button should create a directory 
# in INCOMING_DIR named the button that they want to use.
# they should put their e-mail address in that directory in a
# file called .email
# the administrator directory is named "a"
# and should contain a file .email and .password
# .password should have one line containing a four-number password.







# ============================================================
# ============================================================
# ============================================================
# functions after this!
# ============================================================
# ============================================================
# ============================================================


# ============================================================
# Define the function to receive an answer from the voice library
# ============================================================

function receive
	{
	read -r INPUT <&$VOICE_INPUT;
	echo "$INPUT";
	}

# ============================================================
# Define the function to send a command to the voice library
# ============================================================

function send
	{
	echo $1 >&$VOICE_OUTPUT;
	kill -PIPE $VOICE_PID
	}
	
# ============================================================
# Define the function stop
# ============================================================

function stop
	{
	#logger -t $$ "trying to stop $ANSWER"
	if [ "$ANSWER" == "RECEIVED_DTMF" ]; then
		ANSWER=`receive`; # eat whatever was there.
	fi
	
	#logger -t $$ "just ate $ANSWER"
	send "STOP"
	ANSWER=`receive`;
	
	while [ "$ANSWER" != "READY" ]
	do
		ANSWER=`receive`;
		usleep 1;
	done
	
	#logger -t $$ "stopped with $ANSWER"
	}


# ============================================================
# Define the function stop
# ============================================================

function stopshort
	{

	#logger -t $$ "just ate $ANSWER"
	ANSWER=`receive`;
	
	while [ "$ANSWER" != "READY" ]
	do
		ANSWER=`receive`;
		usleep 4;
	done
	
	#logger -t $$ "stopped with $ANSWER"
	}




# ============================================================
# Define the function send a beep
# ============================================================

function beep
	{
	
	send "BEEP";
	#send "BEEP $1 $2"
	TEMP=`receive`;

	#if [ "$TEMP" != "BEEPING" || "$TEMP" != "READY"]; then
		#logger -t "vgetty[$$]" "Could not start beeping $TEMP";
		#kill -KILL $$
	#	exit_all;
	#fi

	usleep 2;
	TEMP=`receive`

	
	#if [ "$TEMP" != "READY" ]; then
		#echo "$0: Something went wrong on beeping" >&2
		#logger -t "vgetty[$$]" "Not done beeping $TEMP";
	#	exit_all;
	#fi

	}

# ============================================================
# Define the function to play a file
# usage: play filename.rmd
# returns the code that stops it from playing anymore.
# ============================================================

function play
{
	send "PLAY $1"
	#ANSWER=`receive`

	#if [ "$ANSWER" != "PLAYING" ]; then
	#	echo "$0: Could not start playing" >&2
	#	exit_all
	#fi

	while true; do
		ANSWER=`receive`;
		
		case $ANSWER in
			PLAYING)
				ANSWER=`receive`;
					case $ANSWER in
						READY)
							break;
							;;
						RECEIVED_DTMF|BUSY_TONE|FAX_CALLING_TONE|CALL_WAITING)
							
							break;
							;;
						HANDSET_OFF_HOOK|NO_VOICE_ENERGY|SILENCE_DETECTED|DIAL_TONE)
							
							break;
							;;
					esac
				;;
		esac

	done
	
	logger -t $$ "stopped playing for $ANSWER"
	echo "$ANSWER";
}


# ============================================================
# Function to record a message in the $1 named directory.
# returns the full path to the file that was just recorded,
#  or it returns "error" - actually right now it just exits.
# a unique id must be passed as the argument $2 to allow this to name the file.
# call as:
# RECORDED_FILE = `record_dir userid uniqueid`
# ============================================================

function record_dir
	{

	
	logger -t "vgetty[$$]" "about to record a message for DTMF user $1"
	#first, stop other processing.
	stop;
	
	
	#calculate a max length for this message- five minutes.
	CURRTIME=`date +%s`;
	ENDTIME=$(($CURRTIME+300));
	logger -t "vgetty[$$]" "will record from $CURRTIME until $ENDTIME"
	
	# Now that we've played the greeting and read the dtmf
	# code, beep.

	beep;

	ANSWER=`receive`;
	logger -t $$ "right before record, answer is $ANSWER";
	# After beeping, start the recording.  Save it in the user's
	# directory.
	
	send "RECORD $INCOMING_DIR/$1/$2.rmd"
	
	#echo "$0: now starting to record $1/$2.rmd" >&2
	
	#
	# Let's see if it works
	#
	
	#ANSWER=`receive`
	
	#if [ "$ANSWER" != "RECORDING" ]; then
	#	echo "$0: could not start recording" >&2
	#	exit_all
	#fi
	#===
	while true;
	do
		ANSWER=`receive`;
		logger -t $$ "after starting to record, answer is $ANSWER";
		case $ANSWER in
		"RECORDING")
			ANSWER=`receive`;
			
		logger -t $$ "after starting to record2, answer is $ANSWER";
			#if we're recording, keep recording unless time is expired.
			CURRTIME=`date +%s`;
			if [ $(($CURRTIME >= $ENDTIME)) = 1 ]; then
				stop;
				beep;
				RESULT="success";
		#		#echo "$INCOMING_DIR/$1/$STARTING_DATETIME.rmd"
				break;
			fi
			
			
			case $ANSWER in
				READY)
					RESULT=success;
					break;
					;;
				RECEIVED_DTMF)
					stop;
					#beep;
					RESULT=success;
					break;
					;;
				NO_VOICE_ENERGY|SILENCE_DETECTED)
					stopshort;
					RESULT=success;
					break;
					;;
				HANDSET_OFF_HOOK)
					stop;
					beep;
					RESULT=aborted;
					exit_all;
					break;
					;;
			esac
			;;


		esac
	done
	#===
	
	logger -t $$ "I hope that worked... $ANSWER"
	echo "$INCOMING_DIR/$1/$2.rmd"
	}

# ============================================================
# Define the function to end the call completely, no fax or data processing.
# ============================================================

function exit_all
	{
	send "STOP";
	receive;
	send "GOODBYE";
	receive;
	exit 0;
	}

# ============================================================
# Define the function to exit, but send mgetty to fax mode.
# ============================================================

function exit_fax
	{
	send "STOP";
	receive;
	send "GOODBYE";
	receive;
	exit 2;
	}

# ============================================================
# Define the function to exit, but send mgetty to autodetect mode.
# ============================================================

function exit_autodetect
	{
	send "STOP";
	receive;
	send "GOODBYE";
	receive;
	exit 3;
	}


# ============================================================
# Function to get one single DTMF code
# returns the symbol that it read.
# the * symbol is translated to a "q"
# the # symbol is translated to an "a"
# a non-understandable symbol is echoed as is.
#
# you CAN ONLY call this immediately after a "RECEIVED_DTMF"
# was read.
# ============================================================

function one_dtmf
{
	ANSWER=`receive`
	
	echo "$0: first read of DTMF code $ANSWER" >&2;
	
	if [ "$ANSWER" = "*" ]; then
    	ANSWER=q
		TONE=q
	else

		case $ANSWER in
		\#)
			ANSWER=a
			TONE=a
				    ;;
		0|1|2|3|4|5|6|7|8|9)
			TONE=$ANSWER
				    ;;
		*)
			echo "$0: Ignoring DTMF $ANSWER" >&2
			TONE=$ANSWER
				    ;;
		esac

	fi

	echo "$TONE";
}



# ============================================================
# the mail_message function is now basically null, all it does
# is touch the flag file.
# ============================================================

function mail_message
{
	touch $FLAG_FILE;
}

# ============================================================
# a function to mail the rmd file $2 to the user who's id is $1
# converts the .rmd file to a .wav file before mailing
# also chops off the last second.
# after mailing, remove both .wav and .rmd files.
# .wav file is sent as a MIME-encoded attachment.
# returns the name of the file that was mailed.
# call as:
# mail_message dtmf_id /full/path/to/file.rmd
# ============================================================

function mail_message_old
{
	
	#first, stop other processing.
	#stop;
	
	
	#read the e-mail address
	EMAIL_ADDR=`cat $INCOMING_DIR/$1/.email`;
	
	logger -t "vgetty[$$]" "about to convert $2 for $EMAIL_ADDR";
	
	rmdtopvf $2 | pvfcut -T 0.9 | pvfamp -A 2.1 -8 | pvftowav > $INCOMING_DIR/$1/$STARTING_DATETIME.wav;
	
	#MIME-encapsulate and mail the wav file
	mpack -s "Voicemail sent at $STARTING_DATETIME" $INCOMING_DIR/$1/$STARTING_DATETIME.wav $EMAIL_ADDR
	
	SIZ=`ls -la $2`;
	logger -t "vgetty[$$]" "file was $SIZ";
	
	#delete the old files
	rm -f $INCOMING_DIR/$1/$STARTING_DATETIME.rmd
	rm -f $INCOMING_DIR/$1/$STARTING_DATETIME.wav
	
	#logger -t "vgetty[$$]" "just mailed message to user $1 named $INCOMING_DIR/$1/$STARTING_DATETIME.wav"
	
	echo "$INCOMING_DIR/$1/$STARTING_DATETIME.wav"
}




# ============================================================
# Function to wait for any type of message for $1 seconds
# returns the message.
#
#
# THIS IS CURRENTLY BROKEN - DON'T USE IT!
#
# ============================================================

function wait {
	send "WAIT $1";
	ANSWER=`receive`;
	
	while true; do
		case $ANSWER in
			"WAITING")
				#if we're waiting, keep waiting.
				ANSWER=`receive`;
				;;
			"READY")
				#if we're finished waiting, let the user know that they took
				# too long and exit.
				ANSWER=`receive`;
				ANSWER=`play $MESSAGES_DIR/took_too_long.rmd`;
				ANSWER=`play $MESSAGES_DIR/goodbye.rmd`;
				exit_all;
				;;
			*)
				#otherwise, break so that we can return the code that was read.
				break;
				;;
		esac
	done
	echo "$ANSWER";
}





# ============================================================
# The Main Menu
# Let's send the outgoing message if it exists
# while sending the message, listen for any events.
# ============================================================

function main_menu
{

	ANSWER=`play $OUTGOING_DIR/you_have_reached.rmd`
	logger -t $$ "play returned $ANSWER"
	case $ANSWER in
	
		# if READY, then the message finished and we need to wait for something.
		READY) 
			logger -t $$ "here $ANSWER"
			ANSWER=`play $MESSAGES_DIR/took_too_long.rmd`;
			exit_all;
			;;
		
		# if someone hits a button, then we need to figure
		# out what button they pushed.
		RECEIVED_DTMF) main_dtmf;;
		
		#if the person has hung up, then don't do anything.
		BUSY_TONE) stop; exit_all;;
		NO_VOICE_ENERGY) stop; exit_all;;
		SILENCE_DETECTED|HANDSET_OFF_HOOK) stop; exit_all;;
		
		#if it is a fax machine, go to fax mode.
		FAX_CALLING_TONE) stop; exit_fax;;
		
		#anything else is just weird.  Exit.
		*) stop;exit_all;;
	esac

}

# ============================================================
# Main Menu-
# Function to process DTMF button codes for main menu
# ============================================================

function main_dtmf
{

	case $ANSWER in
	"RECEIVED_DTMF")
		# if we got a DTMF, read it and find out what it was.
		ANSWER=`one_dtmf`;
		TONE=$ANSWER;
		echo "$0: read DTMF code $ANSWER, aka $TONE" >&2;
		#verify the received dtmf is allowed
			case $TONE in
			"1") #Jesse Robbins
				RECORDED=`record_dir $TONE $STARTING_DATETIME`;
				#logger -t $$ "record_dir returned $RECORDED";
				#a hack, because I record_dir won't return properly.
				#RECORDED=$INCOMING_DIR/$TONE/$STARTING_DATETIME.rmd;
				mail_message $TONE $RECORDED;
				exit_all;
				;;
			"2") #Ben Brillat
				RECORDED=`record_dir $TONE $STARTING_DATETIME`;
				logger -t $$ "record_dir returned $RECORDED";
				#a hack, because I record_dir won't return properly.
				#RECORDED=$INCOMING_DIR/$TONE/$STARTING_DATETIME.rmd;
				mail_message $TONE $RECORDED;
				exit_all;
				;;
			"3") #Mike Fitzgerald
				RECORDED=`record_dir $TONE $STARTING_DATETIME`;
				#a hack, because I record_dir won't return properly.
				#RECORDED=$INCOMING_DIR/$TONE/$STARTING_DATETIME.rmd;
				mail_message $TONE $RECORDED;
				exit_all;
				;;
			"4") #Matt Schumaker
				RECORDED=`record_dir $TONE $STARTING_DATETIME`;
				mail_message $TONE $RECORDED;
				exit_all;
				;;
			"9") #all of us
				RECORDED=`record_dir $TONE $STARTING_DATETIME`;
				mail_message $TONE $RECORDED;
				exit_all;
				;;
			"q")
				# they pressed the * key to disconnect.
				stopshort;
				exit_all;
				;;
			"a")
				#administrator mode - accessible from the # key.
				#right now all it does is record.
				TEMP=`date +%s`
				RECORDED=`record_dir $TONE $TEMP`;
				main_menu;
				;;
			"0")
				stopshort;
				play $MESSAGES_DIR/credits.rmd;
				exit_all;
				;;
			*)
				#anything else is unacceptable, send a message saying so and restart.
				stopshort;
				play $MESSAGES_DIR/invalid_option.rmd;
				stopshort;
				play $MESSAGES_DIR/disconnect_star.rmd;
				stopshort;
				play $MESSAGES_DIR/main_menu.rmd;
				main_menu;
				;;
			esac
					;;
	*)
		echo "$0: Exit - bad DTMF $ANSWER" >&2;
		exit_all;
					;;
	esac

}


# ============================================================
# the "main" of this program is here...
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# Let's see if the voice library is talking to us
# ============================================================



echo "$0: hello there.  starting a new session at $STARTING_DATETIME" >&2
logger -t "vgetty[$$]" "starting a new session at $STARTING_DATETIME"

ANSWER=`receive`

if [ "$ANSWER" != "HELLO SHELL" ]; then
	echo "$0: voice library not answering" >&2
	exit 1
fi


# Let's answer the message


send "HELLO VOICE PROGRAM"

#
# Let's see if it worked
#

ANSWER=`receive`

if [ "$ANSWER" != "READY" ]; then
	echo "$0: initialization failed" >&2
	exit 1
fi


# Set the device


send "DEVICE DIALUP_LINE"

#
# Let's see if it worked
#

ANSWER=`receive`

if [ "$ANSWER" != "READY" ]; then
	echo "$0: could not set output device" >&2
	exit 1
fi


# enable listening for events.


send "ENABLE EVENTS"
ANSWER=`receive`
if [ "$ANSWER" != "READY" ]; then
	echo "$0: something's wrong enabling events" >&2
	exit 1
fi

#make the thing stop quickly!
send "AUTOSTOP ON";
ANSWER=`receive`;
while [ "$ANSWER" != "READY" ]
do
	ANSWER=`receive`;
	usleep 1;
done
	

#start the thing!
main_menu;



