#!/bin/bash

#
#This is a simple script to answer the phone.
#
# 

# ============================================================
# CONFIGURATION SECTION
# ============================================================


OUTGOING_DIR=/var/spool/voice/messages
INCOMING_DIR=/var/spool/voice/incoming
STARTING_DATETIME=`/bin/date +%Y%b%d_%H-%M-%S%Z`
PATH=$PATH:/usr/local/bin
MESSAGES_DIR=/var/spool/voice/messages
FLAG_FILE=/var/spool/voice/newmail.flag
SCRIPT_DIR=/var/spool/voice/scripts
NUMBER_OF_USERS=5
ZERO=0
SENDMAIL=/usr/sbin/sendmail
SENDER_ADDR=brillb-null@acm.rpi.edu
LOCK_FILE=/var/lock/LCK..ttyS3
LOCK_FILE2=/var/lock/e-vmail

# ============================================================
# a function to mail the rmd file $2 to the user who's id is $1
# converts the .rmd file to a .wav file before mailing
# also chops off the last second.
# after mailing, remove both .wav and .rmd files.
# .wav file is sent as a MIME-encoded attachment.
# returns the name of the file that was mailed.
# call as:
# mail_message dtmf_id /full/path/to/file.rmd
# ============================================================

function mail_message
{
	
	
	
	#read the e-mail address
	EMAIL_ADDR=`cat $INCOMING_DIR/$1/.email`;
	
	logger -t "e-vmail[$$]" "about to convert $2 for $EMAIL_ADDR";
	
	rmdtopvf $2 | pvfcut -T 2 | pvfamp -A 4.0 -8 | pvftowav > $INCOMING_DIR/$1/$2.wav;
	
	#MIME-encapsulate and mail the wav file
	#mpack -s "Voicemail sent at $STARTING_DATETIME" $INCOMING_DIR/$1/$2.wav $EMAIL_ADDR
	mpack -s "Voicemail sent at $STARTING_DATETIME" -o $INCOMING_DIR/$1/$2.mpack $INCOMING_DIR/$1/$2.wav
	
	#cat $INCOMING_DIR/$1/.header $INCOMING_DIR/$1/$2.mpack > $INCOMING_DIR/$1/$2.mail
	
	$SENDMAIL -f$SENDER_ADDR $EMAIL_ADDR < $INCOMING_DIR/$1/$2.mpack
	#/usr/sbin/sendmail -fbrillb-null@acm.rpi.edu $EMAIL_ADDR < $INCOMING_DIR/$1/$2.mpack
	
	SIZ=`ls -la $2.mpack`;
	logger -t "e-vmail[$$]" "file was $SIZ";
	
	#delete the old files
	rm -f $INCOMING_DIR/$1/$2
	rm -f $INCOMING_DIR/$1/$2.wav
	rm -f $INCOMING_DIR/$1/$2.mpack
	rm -f $INCOMING_DIR/$1/$2.mail
	
	echo "$INCOMING_DIR/$1/$2.wav"
	
}


# ============================================================
# this function checks to see if anybody has any mail
# ============================================================
function voicemail_yes_no {

	if [ -f $FLAG_FILE ]; then
		check_lock_file;	# if there is mail, parse the user directories
		rm -f $FLAG_FILE; #after parsing the users, there isn't mail anymore! 
	else
		logger -t "e-vmail[$$]" "no flag file found"
		exit 1;
	fi

}


# ============================================================
# this function checks to see if the lock file exists.
# ============================================================
function check_lock_file {

	if [ ! -f $LOCK_FILE ]; then
		if [ ! -f $LOCK_FILE2 ]; then
			touch $LOCK_FILE2;
			parse_users;	# if there is mail, parse the user directories
			rm -f $LOCK_FILE2;
		else
			logger -t "e-vmail[$$]" "secondary lock file found, aborting"
			exit 1;
		fi
	else
		logger -t "e-vmail[$$]" "lock file found, aborting"
		exit 1;
	fi

}


# ============================================================
# this function goes through the user directories and looks for
# .rmd files.
# ============================================================

function parse_users {

	while [ $(($NUMBER_OF_USERS > $ZERO)) = 1 ];
	do
	  cd $INCOMING_DIR/$NUMBER_OF_USERS
	  NUM_TO_CONVERT=`ls -l *.rmd 2>/dev/null| wc -l | $SCRIPT_DIR/numberecho`
	  for i in *.rmd; do
	   	if [ $(($NUM_TO_CONVERT > $ZERO)) = 1 ]; then
	   		logger -t "e-vmail[$$]" "i is $i num is $NUM_TO_CONVERT"
	   		mail_message $NUMBER_OF_USERS $i;
	   	fi;
	  done;
	  
		NUMBER_OF_USERS=$(($NUMBER_OF_USERS-1));
	done;	



}


# ============================================================
# the "main" of this program is here...
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# ============================================================
# Let's see if the voice library is talking to us
# ============================================================




#logger -t "e-vmail[$$]" "checking for mail at $STARTING_DATETIME";


#start the thing!
#voicemail_yes_no; #use this if we want to check for the existance of
						 			# the flag file first.

check_lock_file; #use this if we just want to filter through the
							# directories looking for files to convert.

rm -f $FLAG_FILE;

exit 1;


